/* 
 * Nazev:	Genealogicky expertni system
 * Tda:	CLI - textove uzivatelske rozhrani
 * Autor:	Antnonin Neumann
 * Datum:	18. 5. 2014
 * Verze:	1
 */
package genealogy;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.Scanner;

public class Cli {
	
	public static Scanner sc = new Scanner(System.in);
	public Lineage tree;
	public static final String EMPHASIS = "===";
	public static final String HELP_STRING = "Napoveda";
	
	/**
	 * Konstruktor textoveho uzivatelskeho rozhrani
	 * 
	 * @param tree Lineage - strom obsahujici rokokmen a pravidla
	 */
	public Cli(Lineage tree){
		this.tree = tree;
	}
	
	/**
	 * Metoda zobrazujici napovedu k prikazum.
	 */
	private void printHelp(){
		System.out.println(EMPHASIS + HELP_STRING + EMPHASIS);
		System.out.println("add \t\t- Pridani osoby nebo vztahu");
		/*System.out.println("delete \t\t- Smazani osoby nebo vztahu");*/
		System.out.println("print \t\t- Vytiskne osoby nebo vztahy");
		System.out.println("ex | expert \t- Umoznuje pokladat systemu otazky");
		System.out.println("h | help \t- Zobrazi napovedu.");
		System.out.println("exit \t\t- Ukonci aplikaci.");
	}
	
	/**
	 * Metoda posloucha prikazi a pote je deleguje.
	 */
	public void listenCmd(){
		System.out.print("> ");
		String cmd = sc.nextLine().trim();
		String[] parts = cmd.split(" ");
		switch (parts[0].toLowerCase()) {
		case "help":
		case "h":
			printHelp();
			break;
		case "add":
			addCmd(parts);
			break;
		case "print":
			printCmd(parts);
			break;
		case "expert":
		case "ex":
			expertCmd(parts);
			break;
		case "exit":
			System.out.println("Ukoncuji aplikaci...");
			System.exit(0);
			break;
		default:
			System.out.println("Spatny prikaz! Pouzijte \"help\" pro ziskani napovedy.");
			break;
		}
		listenCmd();
	}

	/**
	 * Metoda zpracovava prikaz <strong>expert</strong>
	 * 
	 * @param parts String - zadany prikaz
	 */
	private void expertCmd(String[] parts){
		int length = parts.length;
		if(length >= 2){
			Person p, p2;
			Relation r;
			switch (parts[1].toLowerCase()) {
			case "bloodline":
				if(length < 3){
					System.out.println("Nebylo zadno ID osoby!");
					return;
				}
				if((p = this.tree.findPersonById(Integer.parseInt(parts[2]))) != null){
					System.out.println("Nalezena osoba: "+p.toPrint());
					this.tree.printBloodLine(p);
					return;
				}else {
					System.out.println("Osoba nenalezena!");
				}
				break;
			case "person":
				if(length < 4){
					System.out.println("Chybi nektere parametry!");
					return;
				}
				if((p = this.tree.findPersonById(Integer.parseInt(parts[2]))) != null){
					System.out.println("Nalezena osoba: "+p.toPrint());
				}else {
					System.out.println("Osoba nenalezena!");
					return;
				}
				if((r = this.tree.findRelation(Integer.parseInt(parts[3]))) != null){
					System.out.println("Nalezen vztah: "+r.toString());
				}else {
					System.out.println("Vztah nenalezen!");
					return;
				}
				LinkedList<Person> tmp = this.tree.getPerson(p, r);
				if(tmp == null){
					System.out.println(p.getFullName() + " nema " + r.getName(Gender.MALE, true)+"/"+r.getName(Gender.FEMALE, true));
				}else {
					System.out.println(r.getDesc() + " " + p.getFullName() + " je/jsou " + Arrays.toString(tmp.toArray()));
				}
				return;
			case "count":
				if(length < 4){
					System.out.println("Chybi nektere parametry!");
					return;
				}
				if((p = this.tree.findPersonById(Integer.parseInt(parts[2]))) != null){
					System.out.println("Nalezena osoba: "+p.toPrint());
				}else {
					System.out.println("Osoba nenalezena!");
					return;
				}
				if((r = this.tree.findRelation(Integer.parseInt(parts[3]))) != null){
					System.out.println("Nalezen vztah: "+r.toString());
				}else {
					System.out.println("Vztah nenalezen!");
					return;
				}
				System.out.println(p.getFullName() + " ma " + this.tree.countPersonOfRelation(p, r) + " " +
						r.getDesc() + " ("+r.getName(Gender.MALE, true)+"/"+r.getName(Gender.FEMALE, true)+")");
				return;
			case "rel":
				if(length < 4){
					System.out.println("Chybi nektere parametry!");
					return;
				}
				if((p = this.tree.findPersonById(Integer.parseInt(parts[2]))) != null){
					System.out.println("Nalezena osoba: "+p.toPrint());
				}else {
					System.out.println("Osoba nenalezena!");
					return;
				}
				if((p2 = this.tree.findPersonById(Integer.parseInt(parts[3]))) != null){
					System.out.println("Nalezena osoba: "+p2.toPrint());
				}else {
					System.out.println("Osoba nenalezena!");
					return;
				}
				System.out.println("__REL__");
				break;
			}
		}
		System.out.println(EMPHASIS + HELP_STRING + "pro prikaz expert"+ EMPHASIS);
		System.out.println("expert person <Person_ID> <Relation_ID> \t- K zadane osobe najde osoby odpovidajici zadanemu vztahu");
		System.out.println("expert count <Person_ID> <Relation_ID> \t\t- K zadane osobe spocita pocet osob, ktere odpovidaji zadanemu vztahu");
		//System.out.println("expert rel <Person_1_ID> <Person_2_ID> \t\t- Najde vztah zadanych osob");
		System.out.println("expert bloodline <Person_ID> \t\t\t- Vypise pokrevni pribuzne od zadane osoby");
	}
	
	/**
	 * Metoda zpracovava prikaz <strong>add</strong>
	 * 
	 * @param parts String - zadany prikaz
	 */
	private void addCmd(String[] parts){
		int length = parts.length;
		if(length >= 2){
			if(parts[1].equalsIgnoreCase("person")){
				if(length >= 9){
					this.tree.addPerson(parts[2]+";"+parts[3]+";"+parts[4]+";"+parts[5]+";"+parts[6]+";"+parts[7]+";"+parts[8]);
				}else {
					System.out.println("Spatny pocet argumentu pro pridani osoby!");
					System.out.println("add person ID Jmeno Prijmeni Gender ID_matky ID_otce ID_partnera");
				}
				return;
			}else if(parts[1].equalsIgnoreCase("rel")){
				if(length >= 9){
					if (parts[8].matches("^[PCN]+$")){
						if(this.tree.addRelation(new Relation(Integer.parseInt(parts[2]), parts[4], parts[5], parts[6], parts[7], parts[3], parts[8])))
							return;
						else
							System.out.println("Vztah jiz existuje!");
					}else {
						System.out.println("Pravidlo obsahuje nepovolene znaky!");
					}
				}else {	
					System.out.println("Spatny pocet argumentu pro pridani vztahu!");
				}
				System.out.println("add rel ID Description Name_male Name_male_4th Name_female Name_female_4th Rule");
				return;
			}else {
				System.out.println("Spatne zadany prikaz pro pridani!");
			}
		}
		System.out.println(EMPHASIS + HELP_STRING + "pro prikaz add"+ EMPHASIS);
		System.out.println("add person \t- Prida novou osobu");
		System.out.println("add rel \t- Prida novy vztah");
	}
	
	/**
	 * Metoda zpracovava prikaz <strong>print</strong>
	 * 
	 * @param parts String - zadany prikaz
	 */
	private void printCmd(String[] parts){
		if(parts.length >= 2){
			if(parts[1].equalsIgnoreCase("tree")){
				this.tree.printLineage(this.tree.root);
				return;
			}else if(parts[1].equalsIgnoreCase("rel")){
				this.tree.printRules();
				return;
			}else {
				System.out.println("Spatne zadany prikaz pro vypis!");
			}
		}
		System.out.println(EMPHASIS + HELP_STRING + "pro prikaz print"+ EMPHASIS);
		System.out.println("print tree\t- Vypise rodiny strom");
		System.out.println("print rel\t- Vypise vsechny dosputne vztahy");
	}
}
