/* 
 * Nazev:	Genealogicky expertni system
 * Tda:	Main
 * Autor:	Antnonin Neumann
 * Datum:	13. 4. 2014
 * Verze:	1
 */
package genealogy;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;

/**
 * Vstupni trida genealogickeho expertniho systemu
 * 
 * @author Antonin Neumann
 *
 */
public class Main {
	
	public static File rules = new File("rules.ini");
	public static File data = new File("data.ini");
	public static BufferedReader reader;

	/**
	 * Vstupni metoda aplikace
	 * 
	 * @param args String - vstupni argumenty
	 */
	public static void main(String[] args) {
		if(args.length > 1){
			handleArgs(args);
		}else {
			System.out.println("Zadne vstupni argumenty, aplikace pouzije implicitni datasety.");
		}
		
		//nacteni dat do rodokmenu
		String[] loadedData = loadData();
		Lineage tree = new Lineage();
		for (String dataLine : loadedData) {
			tree.addPerson(dataLine);
		}

		//nacteni pravidel pro expertni system
		String[] loadedRules = loadRules();
		for (String dataLine : loadedRules) {
			tree.addRelation(dataLine);
		}
		
		System.out.println("\n===Full family tree===");
		tree.printLineage(tree.root);
		System.out.println("\n===Rules of relation===");
		tree.printRules();
		
		
		
		new Cli(tree).listenCmd();
	}

	/**
	 * Nacte vsechny pripadne vstupni argumenty
	 * 
	 * @param args String - vstupni argumenty
	 */
	public static void handleArgs(String[] args){
		for(int i = 1; i <= args.length; i += 2){
			if(args[i].equalsIgnoreCase("-d") && args[i+1] != null){
				data = new File(args[i+1]);
			}else if(args[i].equalsIgnoreCase("-r") && args[i+1] != null){
				rules = new File(args[i+1]);
			}
		}
	}
	
	/**
	 * Nacte pravidla pro vyhodnoceni rodinych vztahu ze souboru a ulozi je do struktury
	 */
	public static String[] loadRules(){
		/* structure: 
		 * 		ID;NAME;MALE_MALE;MALE_4TH_NAME;FEMALE_MALE;FEMALE_4TH_NAME;RULE;
		 */
		LinkedList<String> s = new LinkedList<String>();
		String[] result;
		try {
		    reader = new BufferedReader(new FileReader(rules));

		    String line;
		    while ((line = reader.readLine()) != null) {
		        s.add(line.substring(1, line.length()-1));
		    }
		    result = new String[s.size()];
		    s.toArray(result);
		    return result;

		} catch (IOException e) {
		    e.printStackTrace();
		} finally {
		    try {
		        reader.close();
		    } catch (IOException e) {
		        e.printStackTrace();
		    }
		}
		return null;
	}
	
	/**
	 * Nacte data o rodinych vztazich a vytvori genealogicky strom
	 */
	public static String[] loadData(){
		/* structure: 
		 * 		ID;NAME;LASTNAME;GENDER;MOTHERID;FATHERID;PARTNERID;
		 */
		LinkedList<String> s = new LinkedList<String>();
		String[] result;
		try {
		    reader = new BufferedReader(new FileReader(data));

		    String line;
		    while ((line = reader.readLine()) != null) {
		        s.add(line.substring(1, line.length()-1));
		    }
		    result = new String[s.size()];
		    s.toArray(result);
		    return result;

		} catch (IOException e) {
		    e.printStackTrace();
		} finally {
		    try {
		        reader.close();
		    } catch (IOException e) {
		        e.printStackTrace();
		    }
		}
		return null;
	}
	
}
