/* 
 * Nazev:	Genealogicky expertni system
 * Tda:	Vztahy
 * Autor:	Antnonin Neumann
 * Datum:	27. 4. 2014
 * Verze:	1
 */
package genealogy;

/**
 * Trida reprezentujici pravidlo pro rodine vztahy.
 * 
 * @author Antonin Neumann
 *
 */
public class Relation {
	private int id;
	private MFString name;
	private String description;
	private String rule;
	
	public Relation(int id, String maleName, String femaleName, String desc, String rule){
		this(id, maleName, null, femaleName, null, desc, rule);
	}
	
	public Relation(int id, String maleName, String maleName7th, String femaleName, String femaleName7th, String desc, String rule){
		this(id, new MFString(maleName, maleName7th, femaleName, femaleName7th), desc, rule);
	}
	
	public Relation(int id, MFString name, String desc, String rule){
		this.id = id;
		this.name = name;
		this.description = desc;
		this.rule = rule;
	}

	public String getRule(){ return this.rule; }
	public int getId() { return this.id; }
	public String getDesc() { return this.description; }

	public String getName(Gender sex, boolean fourth){
		if(fourth){
			return name.get4thName(sex);
		}else {
			return name.getName(sex);
		}
	}
	
	
	
	@Override
	public String toString() {
		return "" +id+ " " + description +
				", rule = " +rule;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		result = prime * result + ((rule == null) ? 0 : rule.hashCode());
		return result;
	}

	/**
	 * Dva vztahy se rovnaji pokud maji stejna pravidla NEBO stejne ID
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Relation other = (Relation) obj;
		if (id == other.id)
			return true;
		if (rule == null) {
			if (other.rule != null)
				return false;
		} else if (!rule.equals(other.rule))
			return false;
		return true;
	}
	
		
}
